"""
This module contains the classes and functions for creating demographics files
for HIV simulations. For more information on |EMOD_s| demographics files,
see :doc:`emod-hiv:software-demographics`. 
"""

import emod_api.demographics.Demographics as Demog
from emod_api.demographics import DemographicsTemplates as DT
from emodpy_hiv.demographics import DemographicsTemplates as HIV_DT

# the mortality algo has a few options. Might want to make them public module variables
# rather than fill up the function with them.

class HIVDemographics(Demog.Demographics):
    """
    This class is derived from :py:class:`emod_api:emod_api.demographics.Demographics.Demographics` 
    and sets certain defaults for HIV in construction.

    Args:
        nodes: The number of nodes to create.
        idref: Method describing how the latitude and longitude values are created
            for each of the nodes in a simulation. "Gridded world" values use a grid 
            overlaid across the globe at some arcsec resolution. You may also generate 
            the grid using another tool or coordinate system. For more information,
            see :ref:`emod-hiv:demo-metadata`.
        base_file: A basic demographics file used as a starting point for
            creating more complicated demographics files. For example,
            using a single node file to create a multi-node file for spatial
            simulations. 
        init_prev: The initial HIV prevalence of the population.

    Returns: 
        None 
     """
    def __init__(self, nodes, idref="Gridded world grump2.5arcmin", base_file=None ):
        super().__init__( nodes, idref, base_file )
        #super().SetDefaultProperties()
        super().SetDefaultNodeAttributes(birth=True)
        HIV_DT.AddDefaultSociety( self )

    def fertility( self, path_to_csv ):
        """
        Set fertility based on data. Simulation shall concist of individual pregnancies with rates by 
        woman's age and year-of-simulation using data from provided csv.
        """
        super().SetDefaultFromTemplate( DT.get_fert_dist( path_to_csv ), DT._set_fertility_age_year )

    def mortality( 
                self,
                file_male,
                file_female,
                interval_fit=[1970, 1980],
                which_point='mid',
                predict_horizon=2050,
                csv_out=False,
                n=0, # I don't know what this means
                results_scale_factor=1.0/365.0):
        """
        For back-compat. This function can go away.
        """

        super().infer_natural_mortality(
                #self,
                file_male,
                file_female,
                interval_fit=interval_fit,
                which_point=which_point,
                predict_horizon=predict_horizon,
                csv_out=csv_out,
                n=n,
                results_scale_factor=results_scale_factor)

    def apply_assortivity( self, category, rowmale_colfemale_matrix ):
        """
            Add an assortivity matrix to Pair-Forming Algo. Right now only applies to RISK IP.

            Args:
                category: "COMMERCIAL", "INFORMAL", "MARITAL", or "TRANSITORY"
                rowmale_colfemale_matrix: 3x3 matrix of assortivity values, row represents male, column represents female.

            Returns:
                N/A.
        """

        if len(rowmale_colfemale_matrix) != 3:
            raise ValueError( "You need to have 3 rows in your assortivity matrix." )
        if any( len(row) != 3 for row in rowmale_colfemale_matrix ):
            raise ValueError( "You need to have 3 columns in each row of your assortivity matrix." )
        assortivity = {}
        assortivity["Axes"] = [ "LOW", "MEDIUM", "HIGH" ]
        assortivity["Group"] = "INDIVIDUAL_PROPERTY"
        assortivity["Property_Name"] = "Risk"
        assortivity["Weighting_Matrix_RowMale_ColumnFemale" ] = rowmale_colfemale_matrix
        self.raw['Defaults']["Society"][category]["Pair_Formation_Parameters"]["Assortivity"].update( assortivity )


def from_template_node(lat=0, lon=0, pop=1e6, name=1, forced_id=1 ):
    """
    Create a single-node :py:class:`~emodpy_hiv.demographics.HIVDemographics`
    instance from the parameters you supply.

    Args:
        lat: Latitude of the centroid of the node to create.
        lon: Longitude of the centroid of the node to create.
        pop: Human population of the node. 
        name: The name of the node. This may be a characteristic of the 
            node, such as "rural" or "urban", or an identifying integer.
        forced_id: The node ID for the single node.

    Returns:
        A :py:class:`~emodpy_malaria.demographics.HIVDemographics` instance.
    """
    new_nodes = [ Demog.Node(lat=lat, lon=lon, pop=pop, name=name, forced_id=forced_id) ] 
    return HIVDemographics(nodes=new_nodes)

def from_pop_csv( pop_filename_in, pop_filename_out="spatial_gridded_pop_dir", site="No_Site" ):
    """
    Create a multi-node :py:class:`~emodpy_hiv.demographics.HIVDemographics`
    instance from a CSV file describing a population.

    Args:
        pop_filename_in: The path to the demographics file to ingest.
        pop_filename_out: The path to the file to output.
        site: A string to identify the country, village, or trial site.

    Returns:
        A :py:class:`~emodpy_malaria.demographics.HIVDemographics` instance.
    """
    generic_demog = Demog.from_pop_csv( pop_filename_in, pop_filename_out, site )
    nodes = generic_demog.nodes
    return HIVDemographics(nodes=nodes, idref=site)

def from_params(tot_pop=1e6, num_nodes=100, frac_rural=0.3, id_ref="from_params" ):
    """
    Create a multi-node :py:class:`~emodpy_hiv.demographics.HIVDemographics`
    instance as a synthetic population based on a few parameters.

    Args:
        tot_pop: The total human population in the node.
        num_nodes: The number of nodes to create.
        frac_rural: The fraction of the population that is rural.
        id_ref: Method describing how the latitude and longitude values are created
            for each of the nodes in a simulation. "Gridded world" values use a grid 
            overlaid across the globe at some arcsec resolution. You may also generate 
            the grid using another tool or coordinate system. For more information,
            see :ref:`emod-hiv:demo-metadata`.

    Returns:
        A :py:class:`~emodpy_hiv.demographics.HIVDemographics` instance.
    """
    generic_demog = Demog.from_params(tot_pop, num_nodes, frac_rural, id_ref )
    nodes = generic_demog.nodes
    return HIVDemographics(nodes=nodes, idref=id_ref )

